/*
*  This program is free software: you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation, either version 3 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
*  Copyright (c) 2011 digitalSTROM.org, Zurich, Switzerland
*  Author: Andreas Brauchli <andreas.brauchli@aizo.com>
*/

/* globals dss, Ext, _, unescape */
// 'use strict';
/* jshint esversion: 6 */

function getParameterByName (name) {
    const regexName = name.replace(/[\[]/, '\\\[').replace(/[\]]/, '\\\]');
    const regexString = '[\\?&]' + regexName + '=([^&#]*)';
    const regex = new RegExp(regexString);
    const results = regex.exec(window.location.search);
    return results && results.length > 0
        ? decodeURIComponent(results[1].replace(/\+/g, ' '))
        : '';
}

function getStateGridTextActive (scene) {
    const sceneName = dss.staticDataModel.getNameForScene(0, 0, scene);
    return Ext.String.format2(_('only when {name} is active'), { name: sceneName });
}

function getStateGridTextInactive (scene) {
    const sceneName = dss.staticDataModel.getNameForScene(0, 0, scene);
    return Ext.String.format2(_('only when {name} is inactive'), { name: sceneName });
}

Ext.define('DSS.addon.SceneResponder', {

    extend: 'DSS.addon.Framework',

    config: {
        appName: 'SceneResponder',
        appId: 'system-addon-scene-responder',
        appVersion: '1.4.6',
        appIcon: 'images/app_icon.png'
    },

    getEntryIds: function (entries) {
        if (entries.get) {
            return entries.get('id');
        }
        const ids = [];
        for (let i = 0; i < entries.length; i++) {
            if (entries[i] && entries[i].get instanceof Function) {
                ids.push(entries[i].get('id'));
            }
        }
        return ids;
    },

    getHelp: function () {
        return Ext.create('Ext.Component', {
            whiteSpace: 'normal',
            padding: '0 20 0 0', // top right bottom left
            loader: {
                url: 'locale/' + dss.staticDataModel.activeLanguage + '/help.html',
                autoLoad: true,
                disableCaching: false
            }
        });
    },

    getContent: function () {
        const me = this;
        /* Parameter:
        *   * editor config
        *   * button config
        *   * store config
        *   * columns
        *   * defaultSortCol
        * */

        const duskTime = dss.staticDataModel.getDusk();
        const dawnTime = dss.staticDataModel.getDawn();
        const sunriseTime = dss.staticDataModel.getSunrise();
        const sunsetTime = dss.staticDataModel.getSunset();
        const conditions = [
            { type: 'singleItem', title: _('state_daynight'), entry: [
                { type: 'state', name: 'daylight', value: true, title: Ext.String.format2(_('daynight_bright'), {
                    duskTime: duskTime,
                    dawnTime: dawnTime
                }), gridText: Ext.String.format2(_('grid_daynight_twilight_true'), {
                    duskTime: duskTime,
                    dawnTime: dawnTime
                }) },
                { type: 'state', name: 'daylight', value: false, title: Ext.String.format2(_('daynight_dark'), {
                    duskTime: duskTime,
                    dawnTime: dawnTime
                }), gridText: Ext.String.format2(_('grid_daynight_twilight_false'), {
                    duskTime: duskTime,
                    dawnTime: dawnTime
                }) },
                { type: 'state', name: 'daynight', value: true, title: Ext.String.format2(_('daynight_day'), {
                    sunriseTime: sunriseTime,
                    sunsetTime: sunsetTime
                }), gridText: Ext.String.format2(_('grid_daynight_daynight_true'), {
                    sunriseTime: sunriseTime,
                    sunsetTime: sunsetTime
                }) },
                { type: 'state', name: 'daynight', value: false, title: Ext.String.format2(_('daynight_night'), {
                    sunriseTime: sunriseTime,
                    sunsetTime: sunsetTime
                }), gridText: Ext.String.format2(_('grid_daynight_daynight_false'), {
                    sunriseTime: sunriseTime,
                    sunsetTime: sunsetTime
                }) } ]
            },
            { type: 'timeframe', title: _('Timeframe') },
            { type: 'weekdays', title: _('Weekdays') },
            { type: 'dateframe', title: _('Dateframe') },
            { type: 'deviceState', title: _('automatic devices') },
            { type: 'vdcDeviceState', title: _('device states') },
            { type: 'userDefinedState', title: _('Custom states') },
            { type: 'globalApplication', title: _('Global Application states') },
            { type: 'singleItem', title: _('state_presence'), entry: [
                { type: 'state', name: 'presence', value: 'present', title: _('presence_present'), gridText: _('grid_presence_present') },
                { type: 'state', name: 'presence', value: 'absent', title: _('presence_absent'), gridText: _('grid_presence_absent') }
            ] },
            { type: 'singleItem', title: _('state_holiday'), entry: [
                { type: 'state', name: 'holiday', value: 'on', title: _('holiday_on'), gridText: _('grid_holiday_on') },
                { type: 'state', name: 'holiday', value: 'off', title: _('holiday_off'), gridText: _('grid_holiday_off') }
            ] },
            { type: 'multiItem', title: _('weather trigger'), entry: [
                { type: 'state', name: 'rain', value: '1', title: _('rain active'), gridText: _('grid_rain_active'), group: 1 },
                { type: 'state', name: 'rain', value: '2', title: _('rain inactive'), gridText: _('grid_rain_inactive'), group: 1 },
                { type: 'state', name: 'wind', value: '1', title: _('wind active'), gridText: _('grid_wind_active'), group: 2 },
                { type: 'state', name: 'wind', value: '2', title: _('wind inactive'), gridText: _('grid_wind_inactive'), group: 2 },
                { type: 'state', name: 'hail', value: '1', title: _('hail active'), gridText: _('grid_hail_active'), group: 3 },
                { type: 'state', name: 'hail', value: '2', title: _('hail inactive'), gridText: _('grid_hail_inactive'), group: 3 },
                { type: 'state', name: 'frost', value: '1', title: _('frost active'), gridText: _('grid_frost_active'), group: 4 },
                { type: 'state', name: 'frost', value: '2', title: _('frost inactive'), gridText: _('grid_frost_inactive'), group: 4 },
                { type: 'state', name: 'sun', value: '1', title: _('sun active'), gridText: _('grid_sun_active'), group: 5 },
                { type: 'state', name: 'sun', value: '2', title: _('sun inactive'), gridText: _('grid_sun_inactive'), group: 5 }
            ] },
            { type: 'state', name: 'service', value: '1', title: _('service active'), gridText: _('grid_service_active'), group: 6 },
            { type: 'state', name: 'service', value: '2', title: _('service inactive'), gridText: _('grid_service_inactive'), group: 6 },
            { type: 'multiItem', title: _('Security'), entry: [
                { type: 'state', name: 'fire', value: '1', title: _('fire active'), gridText: _('grid_fire_active'), group: 1 },
                { type: 'state', name: 'fire', value: '2', title: _('fire inactive'), gridText: _('grid_fire_inactive'), group: 1 },
                { type: 'state', name: 'panic', value: '1', title: _('panic_active'), gridText: _('grid_panic_active'), group: 2 },
                { type: 'state', name: 'panic', value: '2', title: _('panic_inactive'), gridText: _('grid_panic_inactive'), group: 2 },
                { type: 'state', name: 'alarm', value: '1', title: getStateGridTextActive(74), gridText: getStateGridTextActive(74), group: 3 },
                { type: 'state', name: 'alarm', value: '2', title: getStateGridTextInactive(74), gridText: getStateGridTextInactive(74), group: 3 },
                { type: 'state', name: 'alarm2', value: '1', title: getStateGridTextActive(83), gridText: getStateGridTextActive(83), group: 4 },
                { type: 'state', name: 'alarm2', value: '2', title: getStateGridTextInactive(83), gridText: getStateGridTextInactive(83), group: 4 },
                { type: 'state', name: 'alarm3', value: '1', title: getStateGridTextActive(84), gridText: getStateGridTextActive (84), group: 5 },
                { type: 'state', name: 'alarm3', value: '2', title: getStateGridTextInactive(84), gridText: getStateGridTextInactive(84), group: 5 },
                { type: 'state', name: 'alarm4', value: '1', title: getStateGridTextActive(85), gridText: getStateGridTextActive(85), group: 6 },
                { type: 'state', name: 'alarm4', value: '2', title: getStateGridTextInactive(85), gridText: getStateGridTextInactive(85), group: 6 }
            ] },
            { type: 'zonestate', title: _('ZoneState') },
            { type: 'clusterstate', title: _('Cluster State') }
        ];

        const climateSettings = dss.staticDataModel.getClimateSettings();
        if (climateSettings && climateSettings.capabilities && climateSettings.capabilities.coolingSupported) {
            conditions.push({
                type: 'climateState',
                title: _('Climate system'),
                entry: [
                    { type: 'addon-state', name: 'apartment_controller', value: '0', title: _('off'), gridText: _('grid_climate_off'), group: 1 },
                    { type: 'addon-state', name: 'apartment_controller', value: '1', title: _('heating on'), gridText: _('grid_climate_heating'), group: 1 },
                    { type: 'addon-state', name: 'apartment_controller', value: '2', title: _('cooling on'), gridText: _('grid_climate_cooling'), group: 1 }
                ]
            });
        } else {
            conditions.push({
                type: 'climateState',
                title: _('Climate system'),
                entry: [
                    { type: 'addon-state', name: 'apartment_controller', value: '0', title: _('off'), gridText: _('grid_climate_off'), group: 1 },
                    { type: 'addon-state', name: 'apartment_controller', value: '1', title: _('heating on'), gridText: _('grid_climate_heating'), group: 1 }
                ]
            });
        }

        const oConfig = {
            id: 'SRGrid',
            defaultSortCol: 'name',
            disabledProperty: 'disabled',
            inactiveField: 'inactive',
            storeConfig: {
                storeId: 'sceneresponderStore',
                fields: [
                    { name: 'id' },
                    { name: 'disabled', sortType: 'asBoolean' },
                    { name: 'name', sortType: 'asUCText' },
                    { name: 'actions', sortType: 'asAction' },
                    { name: 'triggers', sortType: 'asTrigger' },
                    { name: 'delay', sortType: 'asTimespan' },
                    { name: 'conditions', sortType: 'asCondition' },
                    { name: 'singularTriggered', sortType: 'asBoolean' },
                    { name: 'initialTriggered', sortType: 'asBoolean' },
                    { name: 'doubleChecked', sortType: 'asBoolean' },
                    { name: 'inactive', sortType: 'asBoolean' },
                    { name: 'grouping', sortType: 'asUCText' },
                    { name: 'scope', sortType: 'asUCText' },
                    { name: 'technicalRole', sortType: 'asUCText' }
                ]
            },
            multiselect: true,
            columns: [ {
                header: _('Active'),
                dataIndex: 'disabled',
                resizable: false,
                xtype: 'disabledcolumn',
                width: 50
            }, {
                header: _('Name'),
                dataIndex: 'name',
                xtype: 'textcolumn',
                width: 200
            }, {
                header: _('Trigger'),
                dataIndex: 'triggers',
                xtype: 'triggercolumn',
                maintainFlex: true,
                flex: 3
            }, {
                header: _('Activity'),
                dataIndex: 'actions',
                xtype: 'actioncolumn',
                maintainFlex: true,
                flex: 3
            }, {
                header: _('Conditions'),
                dataIndex: 'conditions',
                xtype: 'conditioncolumn',
                maintainFlex: true,
                flex: 3,
                customRendererOptions: {
                    hideEnabled: true,
                    hideWeekdays: false,
                    hideGenericSystemStates: false,
                    hideZoneStates: false,
                    hideTimeframe: false,
                    filterSystemState: [],
                    stateTitles: [
                        { name: 'daylight', value: true, title: _('grid_daylight_true') },
                        { name: 'daylight', value: false, title:_('grid_daylight_false') },
                        { name: 'daynight', value: true, title: _('grid_daynight_true') },
                        { name: 'daynight', value: false, title:_('grid_daynight_false') },
                        { name: 'panic', value: '1', title: _('grid_panic_active') },
                        { name: 'panic', value: '2', title: _('grid_panic_inactive') },
                        { name: 'presence', value: 'present', title: _('grid_presence_present') },
                        { name: 'presence', value: 'absent', title: _('grid_presence_absent') },
                        { name: 'fire', value: '1', title: _('grid_fire_active') },
                        { name: 'fire', value: '2', title: _('grid_fire_inactive') },
                        { name: 'hail', value: '1', title: _('grid_hail_active') },
                        { name: 'hail', value: '2', title: _('grid_hail_inactive') },
                        { name: 'wind', value: '1', title: _('grid_wind_active') },
                        { name: 'wind', value: '2', title: _('grid_wind_inactive') },
                        { name: 'rain', value: '1', title: _('grid_rain_active') },
                        { name: 'rain', value: '2', title: _('grid_rain_inactive') },
                        { name: 'frost', value: '1', title: _('grid_frost_active') },
                        { name: 'frost', value: '2', title: _('grid_frost_inactive') },
                        { name: 'alarm', value: '1', title: getStateGridTextActive(74) },
                        { name: 'alarm', value: '2', title: getStateGridTextInactive(74) },
                        { name: 'alarm2', value: '1', title: getStateGridTextActive(83) },
                        { name: 'alarm2', value: '2', title: getStateGridTextInactive(83) },
                        { name: 'alarm3', value: '1', title: getStateGridTextActive(84) },
                        { name: 'alarm3', value: '2', title: getStateGridTextInactive(84) },
                        { name: 'alarm4', value: '1', title: getStateGridTextActive(85) },
                        { name: 'alarm4', value: '2', title: getStateGridTextInactive(85) },
                        { name: 'holiday', value: 'on', title: _('grid_holiday_on') },
                        { name: 'holiday', value: 'off', title: _('grid_holiday_off') },
                        { name: 'heating_water_system', value: '0', title: _('grid_climate_off') },
                        { name: 'heating_water_system', value: '1', title: _('grid_climate_heating') },
                        { name: 'heating_water_system', value: '2', title: _('grid_climate_cooling') }
                    ]
                }
            }, {
                header: _('Delay'),
                dataIndex: 'delay',
                xtype: 'timespancolumn',
                width: 80
            } ],
            buttonText: {
                newButton: 'New automaton',
                editButton: 'Edit automaton',
                enableButton: 'Enable automaton',
                disableButton: 'Disable automaton',
                testButton: 'Test automaton',
                deleteButton: 'Delete automaton'
            },
            editorConfig: {
                title: _('Edit automaton'),
                staticControls: [ {
                    type: 'text',
                    label: _('Name'),
                    id: 'name'
                }, {
                    type: 'hidden',
                    label: 'id',
                    id: 'id'
                } ],
                tabControls: [ {
                    id: 'SREditorTriggerTab',
                    tabLabel: _('Trigger'),
                    tabDescription: _('Choose the triggering activity for the message. You can choose more than one activity as trigger.'),
                    content: [ {
                        type: 'TriggerConfigurator',
                        id: 'triggers',
                        userStates: true,
                        flex: true
                    }, {
                        type: 'timespan',
                        label: _('Delay'),
                        id: 'delay',
                        myID: 'SREditorDelay'
                    }, {
                        type: 'checkbox',
                        label: _('should be executed only once when triggered multiple times within delay at the end'),
                        id: 'singularTriggered',
                        myID: 'SREditorRestartDelayTime'
                    } ]
                }, {
                    id: 'SREditorActivityTab',
                    tabLabel: _('Activity'),
                    tabDescription: _('Choose the activities, which should be executed.'),
                    content: [ {
                        type: 'ActionConfigurator',
                        id: 'actions',
                        flex: true,
                        heatingOperationMode: true,
                        withDelay: true,
                        userStates: true
                    } ]
                }, {
                    id: 'SREditorConditionTab',
                    tabLabel: _('Conditions'),
                    tabDescription: _('Choose the conditions.'),
                    content: [ {
                        type: 'ConditionConfigurator',
                        id: 'conditions',
                        conditions: conditions,
                        flex: true
                    } ]
                } ]

                /* conditions: [
                *      {type: <[enabler,weekdays,timeframe,zonestate]>, <[defaultValue:true]>, <[readonly:true]>}
                *   {type: <[singleItem,multiItem]>, entry: [{type:state,name:'<statename>', value: <['active','inactive']>, defaultValue:[{name,state}]}]
                *   |type: deviceState, <filterFunc:callback>, defaultValue:[{name,state}]
                * ]
                * */

            },
            listeners: {
                loadDataStore: function (grid) {
                    me.loadParameter(grid);
                },
                saveEntry: function (grid, values) {
                    me.waitAndReload(grid, {
                        actions: 'save',
                        value: Ext.JSON.encode(values)
                    });
                },
                enableEntry: function (grid, entry) {
                    const ids = me.getEntryIds(entry);
                    me.waitAndReload(grid, {
                        actions: 'enable',
                        id: Ext.JSON.encode(ids)
                    });
                },
                disableEntry: function (grid, entry) {
                    const ids = me.getEntryIds(entry);
                    me.waitAndReload(grid, {
                        actions: 'disable',
                        id: Ext.JSON.encode(ids)
                    });
                },
                deleteEntry: function (grid, entry) {
                    const ids = me.getEntryIds(entry);
                    me.waitAndReload(grid, {
                        actions: 'delete',
                        id: Ext.JSON.encode(ids)
                    });
                },
                testEntry: function (grid, entry) {
                    dss.ajaxAssyncRequestWithoutAnswer('/json/event/raise', {
                        name: 'action_execute',
                        parameter: 'path=/scripts/system-addon-scene-responder/entries/' + entry.get('id') + ';ignoreConditions=ignoreConditions'
                    });
                }
            }
        };

        if (getParameterByName('mode') === 'expert') {
            oConfig.editorConfig.tabControls[0].content.splice(2, 0, {
                type: 'checkbox',
                title: _('only first'),
                label: _('should be executed only once when triggered multiple times within delay, but delay starts with the first trigger'),
                id: 'initialTriggered'
            });
            oConfig.editorConfig.staticControls.push({
                type: 'text',
                label: _('scope'),
                id: 'scope'
            });
            oConfig.editorConfig.staticControls.push({
                type: 'text',
                label: _('technicalRole'),
                id: 'technicalRole'
            });
            oConfig.editorConfig.tabControls[0].content[0].enableUsergroup = true;
            oConfig.editorConfig.tabControls[0].content[0].ignorePresence = true;
            oConfig.editorConfig.tabControls[1].content[0].withDelay = true;
            oConfig.editorConfig.tabControls[1].content[0].enableUsergroup = true;
            oConfig.editorConfig.tabControls[1].content[0].ignorePresence = true;
            oConfig.editorConfig.tabControls[1].content[0].heatingExtendedOperationMode = true;
            oConfig.editorConfig.tabControls[1].content[0].UMRExtendetFunctions = true;
            oConfig.editorConfig.tabControls[2].content[0].ignorePresence = true;
            oConfig.editorConfig.tabControls[2].content[0].fFeatureEnabled = {
                enabler: true,
                weekdays: true,
                timeframe: true,
                systemstate: true,
                zonestate: true,
                userDefinedStates: true
            };
            oConfig.columns.push({
                header: _('Source App'),
                dataIndex: 'scope',
                xtype: 'textcolumn',
                width: 100
            });
            oConfig.columns.push({
                header: _('Technical Role'),
                dataIndex: 'technicalRole',
                xtype: 'textcolumn',
                width: 100
            });
            oConfig.grouperField = 'grouping';
        } else {
            oConfig.scope = { fieldID: 'scope', value: 'system-addon-scene-responder' };
            oConfig.technicalRole = { fieldID: 'technicalRole', value: 'system' };
        }
        const grid = Ext.create('DSS.gridPanel', oConfig);
        grid.editorWindow.editControls.delay.setMargin('4 0 0 5');
        grid.editorWindow.editControls.singularTriggered.setMargin('0 0 0 5');
        return grid;
    },
    waitAndReload: function (grid, parameter) {
        grid.setStatus(_('Saving ...'));
        grid.setDisabled(true);

        const me = this;
        dss.pushAndQuery('system-addon-scene-responder.config', parameter, 'system-addon-scene-responder.saved', function () {  // eventData
            me.loadParameter(grid);
            return true;
        });
    },
    loadParameter: function (grid) {
        grid.setStatus(_('Loading ...'));
        grid.setDisabled(true);
        const sString = dss.ajaxSyncRequest('/json/property/query', {
            query: '/scripts/system-addon-scene-responder/entries/*(*)/*(*)/*(*)/*(*)/*(*)'
        });
        const result = Ext.JSON.decode(sString);

        const oResultObject = dss.parsePropertyQueryAnswer(result, [
            { name: 'name', type: 'simple' },
            { name: 'id', type: 'simple' },
            { name: 'singularTriggered', type: 'simple' },
            { name: 'initialTriggered', type: 'simple' },
            { name: 'doubleChecked', type: 'simple' },
            { name: 'disabled', type: 'condition/enabled' },
            { name: 'delay', type: 'simple' },
            { name: 'actions', type: 'action' },
            { name: 'conditions', type: 'condition' },
            { name: 'triggers', type: 'trigger' },
            { name: 'scope', type: 'simple' },
            { name: 'technicalRole', type: 'simple' }
        ]);

        for (let i = 0; i < oResultObject.length; i++) {
            oResultObject[i].grouping = unescape(oResultObject[i].name).split(' ')[0];
            oResultObject[i].inactive = false;
        }
        grid.loadParameter(oResultObject);
        grid.setStatus(_('Ready'));
        grid.setDisabled(false);
    }
});

Ext.onReady(function () {
    dss.buildUpLang([ 'locale/{languageSuffix}/scene-responder.po' ]);
    const sceneresponder = Ext.create('DSS.addon.SceneResponder');
    sceneresponder.initPage();
});
